/*
 * MiTCR <http://milaboratory.com>
 *
 * Copyright (c) 2010-2013:
 *     Bolotin Dmitriy     <bolotin.dmitriy@gmail.com>
 *     Chudakov Dmitriy    <chudakovdm@mail.ru>
 *
 * MiTCR is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.milaboratory.core.sequence.util;

import com.milaboratory.core.sequence.nucleotide.NucleotideSequence;

/**
 * Nucleotide sub sequence adapter. Implements all NucleotideSequence methods.
 *
 * @author Bolotin Dmitriy (bolotin.dmitriy@gmail.com)
 * @author Shugay Mikhail (mikhail.shugay@gmail.com)
 */
public class NucleotideSubSequence extends NucleotideSequence {
    private final NucleotideSequence sequence;
    private final int from, length;

    public NucleotideSubSequence(NucleotideSequence sequence, int from, int length) {
        if (from < 0 || from >= sequence.size() || length < 0 || from + length > sequence.size())
            throw new IllegalArgumentException();
        this.sequence = sequence;
        this.from = from;
        this.length = length;
    }

    /**
     * Till the end of sequence.
     *
     * @param sequence
     * @param from
     */
    public NucleotideSubSequence(NucleotideSequence sequence, int from) {
        this(sequence, from, sequence.size() - from);
    }

    @Override
    public byte codeAt(int position) {
        if (position < 0 || position >= length)
            throw new IndexOutOfBoundsException();
        return sequence.codeAt(from + position);
    }

    @Override
    public int size() {
        return length;
    }

    public int getTo() {
        return from + length;
    }

    public NucleotideSequence getInnerSequence() {
        return sequence;
    }

    public int getFrom() {
        return from;
    }
}
