package com.milaboratory.core.sequencing.io.fastq;

import com.milaboratory.core.sequencing.io.fastq.quality.QualityStringFormat;

import java.io.BufferedReader;
import java.io.IOException;

/**
 * This class contains methods to infer quality string format from FASTQ data.
 */
public class QualityFormatGuesser {
    /**
     * According to Wikipedia maximal quality is 41 for Illumina 1.8+ format 2 is added just in case.
     */
    private static final int MAX_QUALITY = 43;
    private static final int MIN_QUALITY = -5;

    public static QualityStringFormat guessFormat(BufferedReader reader, int maxBytes) throws IOException {
        String line;
        int k, chr;
        boolean signal33, signal64;
        int maxReadSize = 0, readSize;
        do {
            readSize = 0;

            for (k = 0; k < 3; ++k)
                if ((line = reader.readLine()) != null)
                    readSize += line.length() + 2;
                else
                    return null;

            line = reader.readLine();
            if (line == null)
                return null;
            readSize += line.length() + 2;

            signal33 = false;
            signal64 = false;

            for (k = line.length() - 1; k >= 0; --k) {
                chr = (int) line.charAt(k);
                signal33 |= (chr - 64) < MIN_QUALITY;
                signal64 |= (chr - 33) > MAX_QUALITY;
            }

            if (signal33 && signal64)
                return null;

            if (signal33)
                return QualityStringFormat.Illumina18u;
            if (signal64)
                return QualityStringFormat.Illumina15;

            maxBytes -= readSize;

            if (maxReadSize < readSize)
                maxReadSize = readSize;

        } while (maxBytes - maxReadSize > 0);

        return null;
    }
}
